<?php
/**
*
* @package phpBB3
* @version $Id: functions_json.php, 1.0.2 2010-05-15 00:2:41 Porutchik $
* @copyright (c) 2010 Porutchik http://forum.aeroion.ru
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

/**
 * Convert a PHP scalar, array, hash or object to JSON scalar/array/hash. 
 * This function is an analog of json_encode(), but it can work correctly with a non-latin symbols 
 * and does not analyze the passed data. 
 * Output format must be fully JSON compatible.
 *  
 * Brief example of use:
 *
 * <code>
 * // convert a complexe value to JSON notation, and send it to the browser
 * $value = array('foo', 'bar', array(1, 2, 'baz'), array(3, array(4)));
 * echo php2json($value);
 *
 * // prints: ["foo","bar",[1,2,"baz"],[3,[4]]]
 * </code>
 *  
 * @param mixed $obj   Any structure to convert to JSON.
 * @return string    JSON equivalent structure.
 */
function php2json($obj)
{
	$json_string = '';
	
	switch (true) 
	{
		case is_object($obj):
			$vars = get_object_vars($obj);
			$json_string = php2json($vars);
			break;

		case (is_array($obj) && sizeof($obj) && (array_keys($obj) !== range(0, sizeof($obj) - 1))) :
			// treat as a JSON object
			$result = array();
			foreach ($obj as $key => $value) 
			{
				$result[] = php2json($key) . ':' . php2json($value);
			}
			$json_string = '{' . implode(',', $result) . '}';
			break;

		case is_array($obj) :
			// treat it like a regular array
			$json_string = '[' . implode(',', array_map('php2json', $obj)) . ']';
			break;

		case is_null($obj) :
			$json_string = 'null';
			break;

		case is_bool($obj) :
			$json_string = ($obj ? 'true' : 'false');
			break;
			
		case is_int($obj) :
		case is_float($obj) :
			// Always use "." for floats.
 			$json_string = utf8_str_replace(',', '.', (string) $obj);
			break;

		default :

            // Other values are converted to strings to avoid indeterminism.
            // PHP's "1" and 1 are equal for all PHP operators, but 
            // JS's "1" and 1 are not. So if we pass "1" or 1 from the PHP backend,
            // we should get the same result in the JS frontend (string).
			
			// Character replacements for JSON.
			static $json_replaces = array(
				array("\\", "/", "\n", "\t", "\r", "\b", "\f", '"'),
                array('\\\\', '\\\\\\/', '\\\n', '\\\t', '\\\r', '\\\b', '\\\f', '\\\\\"')
            );
			
			$json_string = '"' . utf8_str_replace($json_replaces[0], $json_replaces[1], (string) $obj) . '"';
	}

	return $json_string;

}

/**
 * Convert a PHP scalar, array, hash or object to JSON scalar/array/hash
 * and send it to user agent
 *
 * @param mixed $obj   Any structure to convert to JSON.
 * @return 
 */
function send_php2json_to_browser($obj)
{
	// Convert the $obj value to JSON value
	$response_text = php2json($obj);

	// application/xhtml+xml not used because of IE
	@header('Content-type: text/plain; charset=UTF-8');
	@header('Cache-Control: private, no-cache="set-cookie"');
	@header('Expires: 0');
	@header('Pragma: no-cache');
	
	echo $response_text;
}

?>